diff --git a/src/commentscan.h b/src/commentscan.h
index e202f0a..23fdda5 100644
--- a/src/commentscan.h
+++ b/src/commentscan.h
@@ -76,6 +76,9 @@ void groupEnterFile(const char *file,int line);
 void groupLeaveFile(const char *file,int line);
 void groupLeaveCompound(const char *file,int line,const char *name);
 void groupEnterCompound(const char *file,int line,const char *name);
+void groupPushAutoGroup();
+void groupPopAutoGroup();
+
 void openGroup(Entry *e,const char *file,int line);
 void closeGroup(Entry *,const char *file,int line,bool foundInline=FALSE);
 void initGroupInfo(Entry *e);
diff --git a/src/commentscan.l b/src/commentscan.l
index fad09d9..afebd14 100644
--- a/src/commentscan.l
+++ b/src/commentscan.l
@@ -443,6 +443,9 @@ static QStack<Grouping> g_autoGroupStack;
 static int              g_memberGroupId = DOX_NOGROUP;
 static QCString         g_memberGroupHeader;
 static QCString         g_memberGroupDocs;
+
+static QStack<QCString> g_compoundAutoGroupNameStack;
+
 static QCString         g_memberGroupRelates;
 static QCString         g_compoundName;
 
@@ -3009,6 +3012,8 @@ void groupEnterFile(const char *fileName,int)
   g_autoGroupStack.clear();
   g_memberGroupId = DOX_NOGROUP;
   g_memberGroupDocs.resize(0);
+  g_compoundAutoGroupNameStack.setAutoDelete(TRUE);
+  g_compoundAutoGroupNameStack.clear();
   g_memberGroupRelates.resize(0);
   g_compoundName=fileName;
 }
@@ -3028,6 +3033,17 @@ void groupLeaveFile(const char *fileName,int line)
   }
 }
 
+void groupPushAutoGroup() {
+  if (!g_autoGroupStack.isEmpty())
+      g_compoundAutoGroupNameStack.push(new QCString(g_autoGroupStack.top()->groupname));
+  else
+      g_compoundAutoGroupNameStack.push(new QCString(""));
+}
+
+void groupPopAutoGroup() {
+   g_compoundAutoGroupNameStack.pop();
+}
+
 void groupEnterCompound(const char *fileName,int line,const char *name)
 {
   if (g_memberGroupId!=DOX_NOGROUP)
@@ -3152,7 +3170,18 @@ void initGroupInfo(Entry *e)
     //printf("Appending group %s to %s: count=%d entry=%p\n",
     //	g_autoGroupStack.top()->groupname.data(),
     //	e->name.data(),e->groups->count(),e);
-    e->groups->append(new Grouping(*g_autoGroupStack.top()));
+    if (g_compoundAutoGroupNameStack.isEmpty() ||
+        *g_compoundAutoGroupNameStack.top() != g_autoGroupStack.top()->groupname) {
+      e->groups->append(new Grouping(*g_autoGroupStack.top()));
+      printf("Appending group %s to %s: count=%d entry=%p\n",
+    	g_autoGroupStack.top()->groupname.data(),
+    	e->name.data(),e->groups->count(),e);
+    }
+    else {
+      printf("Not appending group %s to %s since parent compound is already a member of the group: count=%d entry=%p\n",
+    	g_autoGroupStack.top()->groupname.data(),
+    	e->name.data(),e->groups->count(),e);
+    }
   }
 }
 
diff --git a/src/pyscanner.l b/src/pyscanner.l
index 9c21d41..ee4980c 100644
--- a/src/pyscanner.l
+++ b/src/pyscanner.l
@@ -1587,6 +1587,7 @@ static void parseCompounds(Entry *rt)
   //printf("parseCompounds(%s)\n",rt->name.data());
   EntryListIterator eli(*rt->children());
   Entry *ce;
+  groupPushAutoGroup();
   for (;(ce=eli.current());++eli)
   {
     if (!ce->program.isEmpty())
@@ -1627,6 +1628,7 @@ static void parseCompounds(Entry *rt)
     }
     parseCompounds(ce);
   }
+  groupPopAutoGroup();
 }
 
 //----------------------------------------------------------------------------
diff --git a/src/scanner.l b/src/scanner.l
index 1fe5667..9f5e4d7 100644
--- a/src/scanner.l
+++ b/src/scanner.l
@@ -6754,6 +6754,8 @@ static void parseCompounds(Entry *rt)
   //printf("parseCompounds(%s)\n",rt->name.data());
   EntryListIterator eli(*rt->children());
   Entry *ce;
+  groupPushAutoGroup();
+
   for (;(ce=eli.current());++eli)
   {
     if (!ce->program.isEmpty())
@@ -6866,6 +6868,8 @@ static void parseCompounds(Entry *rt)
     }
     parseCompounds(ce);
   }
+  groupPopAutoGroup();
+
 }
 
 //----------------------------------------------------------------------------
